classdef Mahony_Filter < handle
    % MAHONY_FILTER --> Algorithm that computes pitch and roll using a
    % quaternion based explicit complementary filter. The algorithm
    % assumes x to be the longitudinal axis, y the vertical axis and z the
    % lateral axis.
    %
    % Adapted from SOH Madgwick (2010), and modified to 
    %
    %   Date Created            Last Modified               Author
    %    24-07-2018              24-07-2018                 Erik Wilmes
    %    

    
    %%  Public Properties
    
    properties (Access = public)
        fs = 100;                   % Sample Period
        Quaternion = [1 0 0 0];     % Initial Quaternion
        Kp = 1;                     % Proportional gain
        Ki = 0.2;                   % Integral gain
        t = 0;                      % time
    end
    
     %% Private properties
    properties (Access = private)
        eInt = [0 0 0];             % integral error
    end    
    
    %% Public Methods
    
    methods (Access = public)
        % function to get input filter parameters
        function obj = Mahony_Filter(varargin)
            for i = 1:2:nargin
                if  strcmp(varargin{i},'fs'), obj.fs = varargin{i+1};
                elseif  strcmp(varargin{i},'Quaternion'), obj.Quaternion = varargin{i+1};
                elseif  strcmp(varargin{i},'Kp'), obj.Kp = varargin{i+1};
                elseif  strcmp(varargin{i},'Ki'), obj.Ki = varargin{i+1};
                else error('Invalid argument');
                end
            end
        end
        
        % Madgwick gradient descent algorithm
        
        function obj = UpdateIMU(obj,GyroData,AccData,MagData)
            % UPDATEIMU --> updates orientation IMU
            %
            % Inputs:     obj -->       current settings and status algorithm
            %             AccData -->   Accelerometer acceleration vector
            %             GyroData -->  Gyroscope angular velocity vector
            %             MagData -->   Magnetic field vector
            %
            % Output:     obj -->       updated status algorithm
            
            %------------- Initialize algorithm -----------------%
            
            % Assign variables to make algorithm readable
            q = obj.Quaternion;                 % Quaternion
            dt = 1/obj.fs;                      % stepsize delta
            t = obj.t;
            
            % Normalize Accelerometer data
            AccData = AccData / norm(AccData);
            
            % Normalize Magnetometer data
            MagData = MagData / norm(MagData);
            
            % magnetometer easurement quaternion
            q_mag = [0 MagData(1) MagData(2) MagData(3)];

            % Reference direction of Earth's magnetic field
            h = quaternProd(q, quaternProd(q_mag, quaternConj(q)));
            h = [0 0.388228086059837  -0.921563320230529  0];
            b = [0 norm([h(2) h(4)]) h(3) 0];
            b = h/norm(h);

            
            %---- Explicit complementary filter step ----%
            
            % Estimated direction gravity
            v = [2 * (q(1)*q(4)+q(2)*q(3));
                q(1)^2-q(2)^2+q(3)^2-q(4)^2;
                2*(q(3)*q(4)-q(1)*q(2))];
            
            % Estimated direction magnetic field
            w = [b(2)*(q(1)^2+q(2)^2-q(3)^2-q(4)^2) + 2*b(3)*(q(1)*q(4)+q(2)*q(3)) + 2*b(4)*(q(2)*q(4)-q(1)*q(3));
                2*b(2)*(q(2)*q(3)-q(1)*q(4)) + b(3)*(q(1)^2-q(2)^2+q(3)^2-q(4)^2) + 2*b(4)*(q(1)*q(2)+q(3)*q(4));
                2*b(2)*(q(1)*q(3)+q(2)*q(4))+2*b(3)*(q(3)*q(4)-q(1)*q(2)) + b(4)*(q(1)^2-q(2)^2-q(3)^2+q(4)^2)];
            
            % Compute error between estimated direction and measured
            % direction of gravity and magnetometer together
            e = cross(AccData, v) + cross(MagData, w); 
            
            % error integration if integral gain is present
            if(obj.Ki > 0)
                obj.eInt = obj.eInt + e * dt;   
            else
                obj.eInt = [0 0 0];
            end
            
            % Apply feedback terms
            GyroData = GyroData + obj.Kp * e + obj.Ki * obj.eInt;  
            
            % corrected angular rate quaternion
            q_gyroCorrect = [0 GyroData(1) GyroData(2) GyroData(3)];
            
            % Compute quaternion rate of change
            qdot = 0.5 * quaternProd(q,q_gyroCorrect);           
            
            %--------------- Update orientation quaternion ------------%
            
            % Integrative step to update orientation quaternion
            q = q + qdot * dt;
            
            % Normalize quaternion
            q = q / norm(q);
            
            % Assign updated quaternion to object
            obj.Quaternion = q;
            
            % update time
            t = t+dt;
            obj.t = t;
        end
    end
end

