%% Create3DModelSensors -
%    Processes IMU data from 5 sensors; pelvis, thighs, and shanks
%    Calculates all relevant kinematic parameters
%
%   Date Created:           Last modified:              Author:
%    18-06-2018               25-02-2020              Erik Wilmes
%
% Last Modification:    Added metadata


%% Clear, close all and add path to necessary functions %%
clear;
close all;

addpath('ReadData');
addpath('ReadData/Quaternion_library')


%% General information section %%

% Fill in the exact timing of the calibration procedures. If these are not
% known, leave empty and the program will ask you to select the calibration procedures.

% Enter (if available) start and end of static calibration  (in seconds)
startStatic = [778.682];
endStatic = [781.866];

% Enter start and end of calibration movement of left leg
startCalibLeftLeg = [783.362];
endCalibLeftLeg = [785.214];

% Enter start and end of calibration movement of left leg
startCalibRightLeg = [785.310];
endCalibRightLeg = [787.220];

% Enter start and end of calibration movement of left leg
startCalibPelvis = [787.952];
endCalibPelvis = [789.766];

% Enter Filter settings
filtSet = 'Madgwick';       % 'Madgwick' is default, can be changed to 'Mahony'

% Madgwick filter settings
beta = 0.043;               % 0.043 is default, a higher number means more acceleration and magnetometer influence

% Orientation initialization settings to allow for quick orientation
% initialization
tInit = 3;          % initiation time (3s is default)            
bInit = 3;          % initiation beta (3 is default)

% Mahony filter settings
Kp = 4;             % proportional gain
Ki = 0.1;           % integral gain

% save filter settings in metaData
metaData.filterSettings.filterType = filtSet;
if filtSet == 'Madgwick'
	metaData.filterSettings.beta = beta;
    metaData.filterSettings.bInit = bInit;
    metaData.filterSettings.tInit = tInit;
elseif filtSet == 'Mahony'
    metaData.filterSettings.Kp = Kp;
    metaData.filterSettings.Ki= Ki;
end

%% Data load section %%

% Select all raw data files, (names must be pelvis.dat, leftThigh.dat etc.)

% Get current folder
currentFolder = pwd;

% get folder with data
dataFolder = [currentFolder '/Example Data'];

% get file paths data files
fP= [dataFolder '/pelvis.DAT'];
fLT =[dataFolder '/leftThigh.DAT'];
fLS = [dataFolder '/leftShank.DAT'];
fRT = [dataFolder '/rightThigh.DAT'];
fRS = [dataFolder '/rightShank.DAT'];


% Load data and compute orientation quaternions
pelvis = getIMUData(beta,tInit,bInit,Kp,Ki,filtSet,fP);
leftThigh = getIMUData(beta,tInit,bInit,Kp,Ki,filtSet,fLT);
leftShank = getIMUData(beta,tInit,bInit,Kp,Ki,filtSet,fLS);
rightThigh = getIMUData(beta,tInit,bInit,Kp,Ki,filtSet,fRT);
rightShank = getIMUData(beta,tInit,bInit,Kp,Ki,filtSet,fRS);

% cut data to get equal size
[pelvis,leftThigh,rightThigh,leftShank,rightShank]=...
    cutData(pelvis,leftThigh,rightThigh,leftShank,rightShank);

% obtain time array
time = pelvis.time;

% get sample frequency
fs = pelvis.fs;

% get number of index
nIndex = length(time);

% save metadata
metaData.sensor.pelvis = pelvis.info;
metaData.sensor.leftThigh = leftThigh.info;
metaData.sensor.rightThigh = rightThigh.info;
metaData.sensor.leftShank = leftShank.info;
metaData.sensor.rightShank = rightShank.info;

%% Calibration section %%

% convert seconds to index numbers of calibration period
iViewStart = 1;
iViewEnd = length(time);

% rotate the orientation of each sensor with the global reference frame
% during the static calibration period
if isempty(startStatic) || isempty(endStatic)
    % get static calibration period if not known before
    [pelvis,iStartStatic,iEndStatic] = calibrateQuaternion(pelvis,iViewStart,iViewEnd);
    startStatic = iStartStatic/fs;
    endStatic = iEndStatic/fs;
    
else
    % convert times of static calibration to index numbers
    iStartStatic = round(startStatic*fs);
    iEndStatic = round(endStatic*fs);
    
    % rotate pelvis to temporary frame
    pelvis = calibrateQuaternion(pelvis,[],[],iStartStatic,iEndStatic);
end

% rotate other segments to temporary frames
leftThigh = calibrateQuaternion(leftThigh,[],[],iStartStatic,iEndStatic);
leftShank = calibrateQuaternion(leftShank,[],[],iStartStatic,iEndStatic);
rightThigh = calibrateQuaternion(rightThigh,[],[],iStartStatic,iEndStatic);
rightShank = calibrateQuaternion(rightShank,[],[],iStartStatic,iEndStatic);


%%%% Obtain the frontal and saggital axes of each body frame %%%%

% get rotation pelvis
if isempty(startCalibPelvis) || isempty(endCalibPelvis)
    % if times of calibration movement is unknown
    [qRotatePelvis, startCalibPelvis, endCalibPelvis] = getRotationPelvis(pelvis.gyroBF,fs,iViewStart,iViewEnd);
else
    % if time of calibration movement is known
    iStartCalibPelvis = round(startCalibPelvis*fs);
    iEndCalibPelvis = round(endCalibPelvis*fs);
    
    qRotatePelvis = getRotationPelvis(pelvis.gyroBF,fs,[],[],iStartCalibPelvis,iEndCalibPelvis);
end


% get rotation left leg
if isempty(startCalibLeftLeg) || isempty(endCalibLeftLeg)
    % if times of calibration movement is unknown
    [qRotateLeft, startCalibLeftLeg, endCalibLeftLeg] = getRotation(leftThigh.gyroBF,fs,iViewStart,iViewEnd);
else
    % if time of calibration movement is known
    iStartCalibLeftLeg = round(startCalibLeftLeg*fs);
    iEndCalibLeftLeg = round(endCalibLeftLeg*fs);
    
    qRotateLeft = getRotation(leftThigh.gyroBF,fs,[],[],iStartCalibLeftLeg,iEndCalibLeftLeg);
end

 
% get rotation right leg
if isempty(startCalibRightLeg) || isempty(endCalibRightLeg)
    % if times of calibration movement is unknown
    [qRotateRight, startCalibRightLeg, endCalibRightLeg] = getRotation(rightThigh.gyroBF,fs,iViewStart,iViewEnd);
else
    % if time of calibration movement is known
    iStartCalibRightLeg = round(startCalibRightLeg*fs);
    iEndCalibRightLeg = round(endCalibRightLeg*fs);
    
    qRotateRight = getRotation(rightThigh.gyroBF,fs,[],[],iStartCalibRightLeg,iEndCalibRightLeg);
end


% rotate sensor coordinate frame to the body frame of each sensor
pelvis = gf2bf(pelvis,qRotatePelvis);
leftThigh = gf2bf(leftThigh,qRotateLeft);
leftShank = gf2bf(leftShank,qRotateLeft);
rightThigh = gf2bf(rightThigh,qRotateRight);
rightShank = gf2bf(rightShank,qRotateRight);

% save metadata
metaData.calibration.startStatic = startStatic;
metaData.calibration.endStatic = endStatic;

metaData.calibration.startCalibPelvis = startCalibPelvis;
metaData.calibration.endCalibPelvis = endCalibPelvis;

metaData.calibration.startCalibLeftLeg = startCalibLeftLeg;
metaData.calibration.endCalibLeftLeg = endCalibLeftLeg;

metaData.calibration.startCalibRightLeg = startCalibRightLeg;
metaData.calibration.endCalibRightLeg = endCalibRightLeg;

%% Calculate relative joint quaternions

% pre allocate (saves execution time)
leftThigh.quaternion = zeros(nIndex,4);
leftShank.quaternion = zeros(nIndex,4);
rightThigh.quaternion = zeros(nIndex,4);
rightShank.quaternion = zeros(nIndex,4);

% quaternion left thigh with respect to pelvis (orientation of left hip)
leftThigh.qRelPel = quatmultiply(quatinv(pelvis.qBF),leftThigh.qBF);

% Quaternion left shank with respect to thigh (orientation left knee)
leftShank.qRelThigh = quatmultiply(quatinv(leftThigh.qBF),leftShank.qBF);

% Quaternion right thigh with respect to pelvis (orientation right hip)
rightThigh.qRelPel = quatmultiply(quatinv(pelvis.qBF),rightThigh.qBF);

% Quaternion right shank with respect to thigh (orientation right knee)
rightShank.qRelThigh = quatmultiply(quatinv(rightThigh.qBF),rightShank.qBF);

%% get euler angles and angular velocities joints

% pre-allocate memory
leftThigh.xRot = zeros(nIndex,1);
leftThigh.zRot = zeros(nIndex,1);
leftThigh.yRot = zeros(nIndex,1);

leftShank.xRot = zeros(nIndex,1);
leftShank.zRot = zeros(nIndex,1);
leftShank.yRot = zeros(nIndex,1);

rightThigh.xRot = zeros(nIndex,1);
rightThigh.zRot = zeros(nIndex,1);
rightThigh.yRot = zeros(nIndex,1);

rightShank.xRot = zeros(nIndex,1);
rightShank.zRot = zeros(nIndex,1);
rightShank.yRot = zeros(nIndex,1);

leftThigh.angularVel = zeros(nIndex,3);
rightThigh.angularVel = zeros(nIndex,3);
leftShank.angularVel = zeros(nIndex,3);
rightShank.angularVel = zeros(nIndex,3);

% calculate angular velocities
leftThigh.lHipAngVel = -pelvis.gyroBF+quatrotate(quatinv(leftThigh.qRelPel),leftThigh.gyroBF);
rightThigh.rHipAngVel = -pelvis.gyroBF+quatrotate(quatinv(rightThigh.qRelPel),rightThigh.gyroBF);
leftShank.lKneeAngVel = -leftThigh.gyroBF+quatrotate(quatinv(leftShank.qRelThigh),leftShank.gyroBF);
rightShank.rKneeAngVel = -rightThigh.gyroBF+quatrotate(quatinv(rightShank.qRelThigh),rightShank.gyroBF);

% Convert joint quaternions to euler Angles --> ZXY
[leftThigh.zRot, leftThigh.xRot, leftThigh.yRot] = quat2angle(leftThigh.qRelPel,'ZXY');
[rightThigh.zRot, rightThigh.xRot, rightThigh.yRot] = quat2angle(rightThigh.qRelPel,'ZXY');
[leftShank.zRot, leftShank.xRot, leftShank.yRot] = quat2angle(leftShank.qRelThigh,'ZXY');
[rightShank.zRot, rightShank.xRot, rightShank.yRot] = quat2angle(rightShank.qRelThigh,'ZXY');


% convert to degrees
leftThigh.zRot = rad2deg(leftThigh.zRot);
leftThigh.yRot = rad2deg(leftThigh.yRot);
leftThigh.xRot = rad2deg(leftThigh.xRot);

rightThigh.zRot = rad2deg(rightThigh.zRot);
rightThigh.yRot = rad2deg(rightThigh.yRot);
rightThigh.xRot = rad2deg(rightThigh.xRot);

leftShank.zRot = rad2deg(leftShank.zRot);
leftShank.yRot = rad2deg(leftShank.yRot);
leftShank.xRot = rad2deg(leftShank.xRot);

rightShank.zRot = rad2deg(rightShank.zRot);
rightShank.yRot = rad2deg(rightShank.yRot);
rightShank.xRot = rad2deg(rightShank.xRot);

leftThigh.lHipAngVel = rad2deg(leftThigh.lHipAngVel);
rightThigh.rHipAngVel = rad2deg(rightThigh.rHipAngVel);
leftShank.lKneeAngVel = rad2deg(leftShank.lKneeAngVel);
rightShank.rKneeAngVel = rad2deg(rightShank.rKneeAngVel);

%% Compute anatomical data

% right hip 
rightThigh.flexEx = rightThigh.zRot;
rightThigh.flexExVel = rightThigh.rHipAngVel(:,3);

% left hip 
leftThigh.flexEx = leftThigh.zRot;
leftThigh.flexExVel = leftThigh.lHipAngVel(:,3);

% right knee
rightShank.flexEx = -rightShank.zRot;
rightShank.flexExVel = -rightShank.rKneeAngVel(:,3);

% left knee
leftShank.flexEx = -leftShank.zRot;
leftShank.flexExVel = -leftShank.lKneeAngVel(:,3);


%% Group all data
imuData.pelvis = pelvis;
imuData.leftThigh = leftThigh;
imuData.leftShank = leftShank;
imuData.rightThigh = rightThigh;
imuData.rightShank = rightShank;
imuData.metaData = metaData;

%% Get data of sprint only %%

% start and end time sprint in seconds
startSprint = 1316.5;
endSprint = 1321;

% in index numbers
idxStartSprint = round(startSprint*fs);
idxEndSprint = round(endSprint*fs);

% get data of sprint only
sprintData = createSubset(imuData,idxStartSprint, idxEndSprint);

%% Plots

figure;
set(gcf, 'Position', get(0, 'Screensize'));
subplot(2,2,1)
hold on;
title('Hips Flexion/Extension Sprint')
plot(sprintData.pelvis.time,sprintData.leftThigh.flexEx,'LineWidth',1.5)
plot(sprintData.pelvis.time,sprintData.rightThigh.flexEx,'LineWidth',1.5)
xlabel('Time (s)')
ylabel('Angle (^o)')
xlim([0 max(sprintData.pelvis.time)])
ylim([-30 100])
legend('Left','Right')
set(gca,'FontSize',16)
set(gca, 'YGrid', 'on', 'XGrid', 'off')

subplot(2,2,2)
hold on;
title('Knees Flexion/Extension Sprint')
plot(sprintData.pelvis.time,sprintData.leftShank.flexEx,'LineWidth',1.5)
plot(sprintData.pelvis.time,sprintData.rightShank.flexEx,'LineWidth',1.5)
xlabel('Time (s)')
ylabel('Angle (^o)')
xlim([0 max(sprintData.pelvis.time)])
ylim([-10 160])
legend('Left','Right')
set(gca,'FontSize',16)
set(gca, 'YGrid', 'on', 'XGrid', 'off')

subplot(2,2,3)
hold on;
title('Hips Flexion/Extension Velocity Sprint')
plot(sprintData.pelvis.time,sprintData.leftThigh.flexExVel,'LineWidth',1.5)
plot(sprintData.pelvis.time,sprintData.rightThigh.flexExVel,'LineWidth',1.5)
xlabel('Time (s)')
ylabel('Angular Velocity (^o/s)')
xlim([0 max(sprintData.pelvis.time)])
ylim([-1200 1200])
legend('Left','Right')
set(gca,'FontSize',16)
set(gca, 'YGrid', 'on', 'XGrid', 'off')

subplot(2,2,4)
hold on;
title('Knees Flexion/Extension Velocity Sprint')
plot(sprintData.pelvis.time,sprintData.leftShank.flexExVel,'LineWidth',1.5)
plot(sprintData.pelvis.time,sprintData.rightShank.flexExVel,'LineWidth',1.5)
xlabel('Time (s)')
ylabel('Angular Velocity (^o/s)')
xlim([0 max(sprintData.pelvis.time)])
ylim([-1200 1200])
legend('Left','Right')
set(gca,'FontSize',16)
set(gca, 'YGrid', 'on', 'XGrid', 'off')