function [dataCal,iStart,iEnd] = calibrateQuaternion(data,iViewStart,iViewEnd,iStart,iEnd)
%% CALIBRATEQUATERNION -
%  Sets quaternion to be aligned with earth frame during calibration period
%
%   Input:      data --> struct data.- containing IMU data
%
%   Output:     dataCal --> same struct as input with calibrated quaternion
%
%   Date Created            Last Modified               Author
%    25-05-2018              13-03-2019                 Erik Wilmes
%
%   Modifications:
%
%   13-03-2019 --> Added the option to


%% Select calibration period

if nargin == 1 || nargin ==3
    
    checkCalib = 0;
    
    while checkCalib == 0
        figure;
        % set figure to screensize, comment to undo
        set(gcf, 'Position', get(0, 'Screensize'));
        
        hold on;
        title('Zoom to static calibration period, press Enter // Click start static calibration, then click end static calibration')
        xlabel('time in s')
        ylabel('a in m/s^2')
        
        
        if nargin ==3
            xlim([data.time(iViewStart) data.time(iViewEnd)])
        else
            xlim([data.time(1) data.time(end)])
        end
        
        plot(data.time,data.AccData)
        
        set(gca,'FontSize',14)
        hold off;
        zoom on;
        pause();
        zoom off;
        % select calibration period from figure
        [calib,~] = ginput(2);
        
        % close figure
        close
        
        % Calibrate position IMUs on body
        calStart = calib(1);        % start time calibration period
        calEnd = calib(2);          % end time calibration period
        
        % Find index where calibration period starts and ends
        [~,iStart] = min(abs(data.time-calStart));     % start
        [~,iEnd] = min(abs(data.time-calEnd));         % end
        
        
        checkCalib = checkPlot(data,calStart,calEnd);
        
    end
elseif nargin ==5
    
else
    error('Check input arguments')
end

calStart = data.time(iStart);

% Calculate calibration quaternion during calibration period
cal = mean(data.Quaternion(iStart:iEnd,:),1);      % shank

% normalize quaternion
cal = cal / norm(cal);

% take conjugate
calInv = quaternConj(cal);

% convert quaternion to be alligned with global reference frame
qCal = quatmultiply(data.Quaternion,calInv);
qCal = normalize(qCal,2,'norm');

gyroCal = quatrotate(calInv,data.GyroData);
accCal = quatrotate(calInv,data.AccData);

%% Assign calibrated quaternion to data struct
data.qBF = qCal;
data.gyroBF = gyroCal;
data.accBF = accCal;

data.tCal = round(calStart);

dataCal = data;
end
function check = checkPlot(data,calStart,calEnd)

check = [];
s.f = figure;
hold on;
title('Check if calibration was entered correctly? Answer with yes or no.')
plot(data.time,data.AccData)
xlim([calStart-10 calEnd+10])
line([calStart calStart],get(gca,'Ylim'),'Color','k','LineStyle','--','LineWidth',2)
line([calEnd calEnd],get(gca,'Ylim'),'Color','k','LineStyle','--','LineWidth',2)
ylabel('Acceleration m/sˆ2')

s.noButton = uicontrol('Style','PushButton',...
    'Units','Normalized',...
    'Position',[.6 .12 .3 .1],...
    'String','No',...
    'CallBack',{@answerNo});

s.yesButton = uicontrol('Style','PushButton',...
    'Units','Normalized',...
    'Position',[.2 .12 .3 .1],...
    'String','Yes',...
    'CallBack',{@answerYes});
uiwait

    function answerNo(s,event)
        check = 0;
        uiresume
    end

    function answerYes(s,event)
        check = 1;
        uiresume
    end

close(s.f)
end
