function data = getIMUData(beta,tInit,bInit,Kp,Ki,filterMethod,fileName)
%% GETIMUDATA -->
% Function that obtains orientation quaternions in time of raw IMU data
%
% Inputs:       beta    --> Madgwick algorithm filter gain
%               tInit   --> time specifying length initializing period
%                           for fast convergence
%               bInit   --> Madgwick algorithm filter gain during
%                           initializing period
%               fileName --> file and path of file
%
% Outputs:      data --> struct data.-
%                   Quaternion --> Orientation quaternion IMU
%                   fs         --> sample frequency IMU
%                   time       --> corresponding time array
%                   AccData    --> raw accelerometer data
%                   GyroData   --> raw gyroscope data
%                   MagData    --> raw magnetometer data
%
%   Date Created            Last Modified               Author
%    25-05-2018              07-01-2020                Erik Wilmes
%
% Modifications:
%
% 20-03-2019 --> Only delete the locally created .mat file
% 07-01-2020 --> Add the option to not pre-set a filename

%% Load data

% read data
if nargin ==7
    [rawData,f,data.fs,data.info] =  ReadAndSyncSensors2(fileName);
elseif nargin ==6
    [rawData,f,data.fs,data.info] =  ReadAndSyncSensors2();
end

%% Format data

% get accelerometer data
data.AccData = rawData(:,1:3);

% get gyroscope data
data.GyroData = rawData(:,4:6);
[B,A] = butter ( 2, 12/(data.fs/2),'low');    %12 Hz low pass filtering Gyro
data.GyroData = filtfilt(B,A,data.GyroData);

% get magnetometer data
data.MagData = rawData(:,7:9);

% create time array
data.time = linspace(0,size(rawData,1)/data.fs,size(rawData,1)).';

%% Calculate orientation quaternion

% initial quaternion
q_init = [1 0 0 0];

if strcmp(filterMethod,'Madgwick')
    % initialize algorithm
    IMU = Madgwick_Filter('fs', data.fs,'Quaternion',q_init,'Beta', beta,'tInit',tInit,'bInit',bInit);
    
elseif strcmp(filterMethod,'Mahony')
    IMU = Mahony_Filter('fs', data.fs,'Quaternion',q_init,'Kp',Kp,'Ki',Ki);
    
end

% pre - allocat
q = zeros(length(data.time),4);


for t = 1:length(data.time)
    
    IMU.UpdateIMU(data.GyroData(t,:) , data.AccData(t,:),data.MagData(t,:));
    
    % save quaternion
    q(t, :) = IMU.Quaternion;
    
end

% assign q to struct
data.Quaternion = q;


%% Delete created .mat files
name = [f '.mat'];
delete(name)
